#!/bin/bash
# Script to collect system and process data for monitoring.
#
# Author: Xavi Lleó
# Copyright (c) 2025 Verdnatura S.L. All rights reserved.
# Version: 1.0.0
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This script collects system statistics and process information
# and logs it to a file in /var/log/processes. It includes:
#   - CPU and memory usage with `top`
#   - Process queue statistics with `sar`
#   - Nagios process count check
#   - MySQL process list
#
# Execution:
# The script is designed to be executed by a systemd timer every 5 minutes
# between 02:30 and 06:30 AM daily. The timer is configured to run the
# script located at /usr/local/bin/monitor_processes.sh.
#
# Log Location:
# The log file will be stored in /var/log/processes with a timestamp in the filename.
#
# Example Timer Configuration:
# The following systemd timer will execute the script every 5 minutes
# between 02:30 and 06:30 AM:
# 
# [Timer]
# OnCalendar=*-*-* 02:30:00
# Persistent=true
# OnBootSec=10min
# OnUnitActiveSec=5min
# Until=*-*-* 06:30:00
#
# [Service]
# Type=oneshot
# ExecStart=/usr/local/bin/monitor_processes.sh
#
# Ensure that the systemd timer and service are enabled to run as expected.

LOG_DIR="/var/log/processes"
mkdir -p "$LOG_DIR"

timestamp() {
    date +'%Y_%m_%d-%H:%M:%S'
}

LOG_FILE="$LOG_DIR/processes_$(timestamp).log"

log() {
    echo "$1" >> "$LOG_FILE"
}

check_command() {
    command -v "$1" >/dev/null 2>&1
}

log "Process and system data collection - $(timestamp)"
log "-------------------------------------------"

commands=(
    "/usr/bin/top -b -n 1"
    "/usr/bin/sar -q 1 10"
    "/usr/lib/nagios/plugins/check_procs"
    "/usr/bin/mysql -Bse 'show processlist;'"
)

for cmd in "${commands[@]}"; do
    cmd_name=$(echo $cmd | cut -d' ' -f1)
    
    if check_command "$cmd_name"; then
        log "### Executing $cmd_name ###"
        eval $cmd >> "$LOG_FILE" 2>&1
        if [ $? -ne 0 ]; then
            log "Error executing '$cmd_name'"
        fi
    else
        log "Skipping execution of '$cmd_name' due to missing command"
    fi
done

log "-------------------------------------------"
log "Log finished - $(timestamp)"

